@@ -219,21 +219,21 @@ def tokenize_attribute(iterable, attribute):
         atrv = mattr.group(1)
         if r_comattrval.match(atrv):
             name, type = tokenize_single_comma(atrv)
-            next = iterable.next()
+            next_item = iterable.next()
         elif r_wcomattrval.match(atrv):
             name, type = tokenize_single_wcomma(atrv)
-            next = iterable.next()
+            next_item = iterable.next()
         else:
             # Not sure we should support this, as it does not seem supported by
             # weka.
             raise ValueError("multi line not supported yet")
-            #name, type, next = tokenize_multilines(iterable, atrv)
+            #name, type, next_item = tokenize_multilines(iterable, atrv)
     else:
         raise ValueError("First line unparsable: %s" % sattr)
 
     if type == 'relational':
         raise ValueError("relational attributes not supported yet")
-    return name, type, next
+    return name, type, next_item
 
 
 def tokenize_multilines(iterable, val):
@@ -343,7 +343,7 @@ def safe_float(x):
     >>> safe_float('?\\n')
     nan
     """
-    if x.strip() == '?':
+    if '?' in x:
         return np.nan
     else:
         return np.float(x)
@@ -450,11 +450,19 @@ class MetaData(object):
 
     def types(self):
         """Return the list of attribute types."""
-        return [v[0] for v in self._attributes.values()]
+        attr_types = [self._attributes[name][0] for name in self._attrnames]
+        return attr_types
 
 
 def loadarff(filename):
-    """Read an arff file.
+    """
+    Read an arff file.
+
+    The data is returned as a record array, which can be accessed much like
+    a dictionary of numpy arrays.  For example, if one of the attributes is
+    called 'pressure', then its first 10 data points can be accessed from the
+    ``data`` record array like so: ``data['pressure'][0:10]``
+
 
     Parameters
     ----------
@@ -464,22 +472,32 @@ def loadarff(filename):
     Returns
     -------
     data : record array
-       the data of the arff file. Each record corresponds to one attribute.
-    meta : MetaData
-       this contains information about the arff file, like type and
-       names of attributes, the relation (name of the dataset), etc...
+       The data of the arff file, accessible by attribute names.
+    meta : `MetaData`
+       Contains information about the arff file such as name and
+       type of attributes, the relation (name of the dataset), etc...
+
+    Raises
+    ------
+    `ParseArffError`
+        This is raised if the given file is not ARFF-formatted.
+    NotImplementedError
+        The ARFF file has an attribute which is not supported yet.
 
     Notes
     -----
 
     This function should be able to read most arff files. Not
-    implemented functionalities include:
+    implemented functionality include:
 
     * date type attributes
     * string type attributes
 
-    It can read files with numeric and nominal attributes.  It can read
-    files with sparse data (? in the file).
+    It can read files with numeric and nominal attributes.  It cannot read
+    files with sparse data ({} in the file).  However, this function can
+    read files with missing data (? in the file), representing the data
+    points as NaNs.
+
     """
     ofile = open(filename)
 
@@ -574,15 +592,20 @@ def loadarff(filename):
         while r_comment.match(raw):
             raw = row_iter.next()
 
+        # 'compiling' the range since it does not change
+        # Note, I have already tried zipping the converters and
+        # row elements and got slightly worse performance.
+        elems = range(ni)
+
         row = raw.split(delim)
-        yield tuple([convertors[i](row[i]) for i in range(ni)])
+        yield tuple([convertors[i](row[i]) for i in elems])
         for raw in row_iter:
             while r_comment.match(raw):
                 raw = row_iter.next()
             while r_empty.match(raw):
                 raw = row_iter.next()
             row = raw.split(delim)
-            yield tuple([convertors[i](row[i]) for i in range(ni)])
+            yield tuple([convertors[i](row[i]) for i in elems])
 
     a = generator(ofile, delim = delim)
     # No error should happen here: it is a bug otherwise
