@@ -2,12 +2,12 @@
 
 def configuration(parent_package='',top_path=None):
     from numpy.distutils.misc_util import Configuration
-    config = Configuration('io', parent_package, top_path)
+    config = Configuration('io', parent_package, top_path,
+                           setup_name = 'setupscons.py')
 
     config.add_sconscript('SConstruct')
 
     config.add_data_dir('tests')
-    config.add_data_dir('examples')
     config.add_data_dir('docs')
     config.add_subpackage('matlab')
     config.add_subpackage('arff')
