@@ -3,11 +3,9 @@
 #
 # w/ additions by Travis Oliphant, March 2002
 
-__all__ = ['solve', 'solveh_banded', 'solve_banded',
+__all__ = ['solve', 'solve_triangular', 'solveh_banded', 'solve_banded',
             'inv', 'det', 'lstsq', 'pinv', 'pinv2']
 
-from warnings import warn
-
 from numpy import asarray, zeros, sum, conjugate, dot, transpose, \
         asarray_chkfinite,  single
 import numpy
@@ -48,9 +46,9 @@ def solve(a, b, sym_pos=False, lower=False, overwrite_a=False, overwrite_b=False
     """
     a1, b1 = map(asarray_chkfinite,(a,b))
     if len(a1.shape) != 2 or a1.shape[0] != a1.shape[1]:
-        raise ValueError, 'expected square matrix'
+        raise ValueError('expected square matrix')
     if a1.shape[0] != b1.shape[0]:
-        raise ValueError, 'incompatible dimensions'
+        raise ValueError('incompatible dimensions')
     overwrite_a = overwrite_a or (a1 is not a and not hasattr(a,'__array__'))
     overwrite_b = overwrite_b or (b1 is not b and not hasattr(b,'__array__'))
     if debug:
@@ -73,6 +71,66 @@ def solve(a, b, sym_pos=False, lower=False, overwrite_a=False, overwrite_b=False
     raise ValueError('illegal value in %d-th argument of internal gesv|posv'
                                                                     % -info)
 
+def solve_triangular(a, b, trans=0, lower=False, unit_diagonal=False,
+                     overwrite_b=False, debug=False):
+    """Solve the equation `a x = b` for `x`, assuming a is a triangular matrix.
+
+    Parameters
+    ----------
+    a : array, shape (M, M)
+    b : array, shape (M,) or (M, N)
+    lower : boolean
+        Use only data contained in the lower triangle of a.
+        Default is to use upper triangle.
+    trans : {0, 1, 2, 'N', 'T', 'C'}
+        Type of system to solve:
+
+        ========  =========
+        trans     system
+        ========  =========
+        0 or 'N'  a x   = b
+        1 or 'T'  a^T x = b
+        2 or 'C'  a^H x = b
+        ========  =========
+
+    unit_diagonal : boolean
+        If True, diagonal elements of A are assumed to be 1 and
+        will not be referenced.
+
+    overwrite_b : boolean
+        Allow overwriting data in b (may enhance performance)
+
+    Returns
+    -------
+    x : array, shape (M,) or (M, N) depending on b
+        Solution to the system a x = b
+
+    Raises
+    ------
+    LinAlgError
+        If a is singular
+
+    """
+
+    a1, b1 = map(asarray_chkfinite,(a,b))
+    if len(a1.shape) != 2 or a1.shape[0] != a1.shape[1]:
+        raise ValueError('expected square matrix')
+    if a1.shape[0] != b1.shape[0]:
+        raise ValueError('incompatible dimensions')
+    overwrite_b = overwrite_b or (b1 is not b and not hasattr(b,'__array__'))
+    if debug:
+        print 'solve:overwrite_b=',overwrite_b
+    trans = {'N': 0, 'T': 1, 'C': 2}.get(trans, trans)
+    trtrs, = get_lapack_funcs(('trtrs',), (a1,b1))
+    x, info = trtrs(a1, b1, overwrite_b=overwrite_b, lower=lower,
+                    trans=trans, unitdiag=unit_diagonal)
+
+    if info == 0:
+        return x
+    if info > 0:
+        raise LinAlgError("singular matrix: resolution failed at diagonal %s" % (info-1))
+    raise ValueError('illegal value in %d-th argument of internal trtrs')
+
 def solve_banded((l, u), ab, b, overwrite_ab=False, overwrite_b=False,
           debug=False):
     """Solve the equation a x = b for x, assuming a is banded matrix.
@@ -167,19 +225,10 @@ def solveh_banded(ab, b, overwrite_ab=False, overwrite_b=False, lower=False):
 
     Returns
     -------
-    c : array, shape (u+1, M)
-        Cholesky factorization of a, in the same banded format as ab
     x : array, shape (M,) or (M, K)
         The solution to the system a x = b
-        
-    Notes
-    -----
-    The inclusion of `c` in the return value is deprecated.  In SciPy
-    version 0.9, the return value will be the solution `x` only.
 
     """
-    warn("In SciPy 0.9, the return value of solveh_banded will be "
-            "the solution x only.", DeprecationWarning)
 
     ab, b = map(asarray_chkfinite, (ab, b))
 
@@ -195,32 +244,40 @@ def solveh_banded(ab, b, overwrite_ab=False, overwrite_b=False, lower=False):
     if info < 0:
         raise ValueError('illegal value in %d-th argument of internal pbsv'
                                                                     % -info)
-    return c, x
+    return x
 
 
 # matrix inversion
 def inv(a, overwrite_a=False):
-    """Compute the inverse of a matrix.
+    """
+    Compute the inverse of a matrix.
 
     Parameters
     ----------
-    a : array-like, shape (M, M)
-        Matrix to be inverted
+    a : array_like
+        Square matrix to be inverted.
+    overwrite_a : bool, optional
+        Discard data in `a` (may improve performance). Default is False.
 
     Returns
     -------
-    ainv : array-like, shape (M, M)
-        Inverse of the matrix a
+    ainv : ndarray
+        Inverse of the matrix `a`.
 
-    Raises LinAlgError if a is singular
+    Raises
+    ------
+    LinAlgError :
+        If `a` is singular.
+    ValueError :
+        If `a` is not square, or not 2-dimensional.
 
     Examples
     --------
-    >>> a = array([[1., 2.], [3., 4.]])
-    >>> inv(a)
+    >>> a = np.array([[1., 2.], [3., 4.]])
+    >>> sp.linalg.inv(a)
     array([[-2. ,  1. ],
            [ 1.5, -0.5]])
-    >>> dot(a, inv(a))
+    >>> np.dot(a, sp.linalg.inv(a))
     array([[ 1.,  0.],
            [ 0.,  1.]])
 
@@ -351,7 +408,7 @@ def lstsq(a, b, cond=None, overwrite_a=False, overwrite_b=False):
     """
     a1, b1 = map(asarray_chkfinite, (a, b))
     if len(a1.shape) != 2:
-        raise ValueError, 'expected matrix'
+        raise ValueError('expected matrix')
     m, n = a1.shape
     if len(b1.shape) == 2:
         nrhs = b1.shape[1]
@@ -387,7 +444,7 @@ def lstsq(a, b, cond=None, overwrite_a=False, overwrite_b=False):
     if n < m:
         x1 = x[:n]
         if rank == n:
-            resids = sum(x[n:]**2, axis=0)
+            resids = sum(abs(x[n:])**2, axis=0)
         x = x1
     return x, resids, rank, s
 
