@@ -1,10 +1,7 @@
 #!/usr/bin/env python
 
 import os
-import sys
-import re
 from distutils.dep_util import newer_group, newer
-from glob import glob
 from os.path import join
 
 #-------------------
@@ -49,12 +46,12 @@ def configuration(parent_package='',top_path=None):
     lapack_opt = get_info('lapack_opt')
 
     if not lapack_opt:
-        raise NotFoundError,'no lapack/blas resources found'
+        raise NotFoundError('no lapack/blas resources found')
 
     atlas_version = ([v[3:-3] for k,v in lapack_opt.get('define_macros',[]) \
                       if k=='ATLAS_INFO']+[None])[0]
     if atlas_version:
-        print 'ATLAS version',atlas_version
+        print ('ATLAS version: %s' % atlas_version)
 
     target_dir = ''
     skip_names = {'clapack':[],'flapack':[],'cblas':[],'fblas':[]}
@@ -87,7 +84,7 @@ def configuration(parent_package='',top_path=None):
         skip_names['clapack'].extend(\
             'sgetri dgetri cgetri zgetri spotri dpotri cpotri zpotri'\
             ' slauum dlauum clauum zlauum strtri dtrtri ctrtri ztrtri'.split())
-    elif atlas_version>'3.4.0' and atlas_version<='3.5.12':
+    elif atlas_version and atlas_version>'3.4.0' and atlas_version<='3.5.12':
         skip_names['clapack'].extend('cpotrf zpotrf'.split())
 
     def generate_pyf(extension, build_dir):
@@ -175,10 +172,15 @@ def configuration(parent_package='',top_path=None):
                          )
 
     # atlas_version:
+    if os.name == 'nt' and 'FPATH' in os.environ:
+        define_macros = [('NO_ATLAS_INFO', 1)]
+    else:
+        define_macros = []
 
     config.add_extension('atlas_version',
                          ['atlas_version.c'],
-                         extra_info = lapack_opt
+                         extra_info = lapack_opt,
+                         define_macros = define_macros
                          )
 
     config.add_data_dir('tests')
