@@ -3,28 +3,33 @@ import _minpack
 
 from numpy import atleast_1d, dot, take, triu, shape, eye, \
                   transpose, zeros, product, greater, array, \
-                  all, where, isscalar, asarray, inf
+                  all, where, isscalar, asarray, inf, abs
 
 error = _minpack.error
 
-__all__ = ['fsolve', 'leastsq', 'fixed_point', 'bisection', 'curve_fit']
+__all__ = ['fsolve', 'leastsq', 'fixed_point', 'curve_fit']
 
-def check_func(thefunc, x0, args, numinputs, output_shape=None):
-    res = atleast_1d(thefunc(*((x0[:numinputs],)+args)))
+def _check_func(checker, argname, thefunc, x0, args, numinputs, output_shape=None):
+    res = atleast_1d(thefunc(*((x0[:numinputs],) + args)))
     if (output_shape is not None) and (shape(res) != output_shape):
         if (output_shape[0] != 1):
             if len(output_shape) > 1:
                 if output_shape[1] == 1:
                     return shape(res)
-            msg = "There is a mismatch between the input and output " \
-                  "shape of %s." % thefunc.func_name
+            msg = "%s: there is a mismatch between the input and output " \
+                  "shape of the '%s' argument" % (checker, argname)
+            func_name = getattr(thefunc, 'func_name', None)
+            if func_name:
+                msg += " '%s'." % func_name
+            else:
+                msg += "."
             raise TypeError(msg)
     return shape(res)
 
 
 def fsolve(func, x0, args=(), fprime=None, full_output=0,
            col_deriv=0, xtol=1.49012e-8, maxfev=0, band=None,
-           epsfcn=0.0, factor=100, diag=None, warning=True):
+           epsfcn=0.0, factor=100, diag=None):
     """
     Find the roots of a function.
 
@@ -47,9 +52,6 @@ def fsolve(func, x0, args=(), fprime=None, full_output=0,
     col_deriv : bool
         Specify whether the Jacobian function computes derivatives down
         the columns (faster, because there is no transpose operation).
-    warning : bool
-        Whether to print a warning message when the call is unsuccessful.
-        This option is deprecated, use the warnings module instead.
 
     Returns
     -------
@@ -106,33 +108,27 @@ def fsolve(func, x0, args=(), fprime=None, full_output=0,
     -----
     ``fsolve`` is a wrapper around MINPACK's hybrd and hybrj algorithms.
 
-    From scipy 0.8.0 `fsolve` returns an array of size one instead of a scalar
-    when solving for a single parameter.
-
     """
-    if not warning :
-        msg = "The warning keyword is deprecated. Use the warnings module."
-        warnings.warn(msg, DeprecationWarning)
-    x0 = array(x0,ndmin=1)
+    x0 = array(x0, ndmin=1)
     n = len(x0)
     if type(args) != type(()): args = (args,)
-    check_func(func,x0,args,n,(n,))
+    _check_func('fsolve', 'func', func, x0, args, n, (n,))
     Dfun = fprime
     if Dfun is None:
         if band is None:
-            ml,mu = -10,-10
+            ml, mu = -10,-10
         else:
-            ml,mu = band[:2]
+            ml, mu = band[:2]
         if (maxfev == 0):
-            maxfev = 200*(n+1)
+            maxfev = 200*(n + 1)
         retval = _minpack._hybrd(func, x0, args, full_output, xtol,
-                                 maxfev, ml, mu, epsfcn, factor, diag)
+                maxfev, ml, mu, epsfcn, factor, diag)
     else:
-        check_func(Dfun,x0,args,n,(n,n))
+        _check_func('fsolve', 'fprime', Dfun, x0, args, n, (n,n))
         if (maxfev == 0):
-            maxfev = 100*(n+1)
+            maxfev = 100*(n + 1)
         retval = _minpack._hybrj(func, Dfun, x0, args, full_output,
-                                 col_deriv, xtol, maxfev, factor,diag)
+                col_deriv, xtol, maxfev, factor,diag)
 
     errors = {0:["Improper input parameters were entered.",TypeError],
               1:["The solution converged.", None],
@@ -171,8 +167,9 @@ def fsolve(func, x0, args=(), fprime=None, full_output=0,
 
 def leastsq(func, x0, args=(), Dfun=None, full_output=0,
             col_deriv=0, ftol=1.49012e-8, xtol=1.49012e-8,
-            gtol=0.0, maxfev=0, epsfcn=0.0, factor=100, diag=None,warning=True):
-    """Minimize the sum of squares of a set of equations.
+            gtol=0.0, maxfev=0, epsfcn=0.0, factor=100, diag=None):
+    """
+    Minimize the sum of squares of a set of equations.
 
     ::
 
@@ -216,9 +213,6 @@ def leastsq(func, x0, args=(), Dfun=None, full_output=0,
         (``factor * || diag * x||``). Should be in interval ``(0.1, 100)``.
     diag : sequence
         N positive entries that serve as a scale factors for the variables.
-    warning : bool
-        Whether to print a warning message when the call is unsuccessful.
-        Deprecated, use the warnings module instead.
 
     Returns
     -------
@@ -233,7 +227,7 @@ def leastsq(func, x0, args=(), Dfun=None, full_output=0,
         residual standard deviation to get the covariance of the
         parameter estimates -- see curve_fit.
     infodict : dict
-        a dictionary of optional outputs with the keys::
+        a dictionary of optional outputs with the key s::
 
             - 'nfev' : the number of function calls
             - 'fvec' : the function evaluated at the output
@@ -249,6 +243,7 @@ def leastsq(func, x0, args=(), Dfun=None, full_output=0,
                      magnitude. Column j of p is column ipvt(j)
                      of the identity matrix.
             - 'qtf'  : the vector (transpose(q) * fvec).
+
     mesg : str
         A string message giving information about the cause of failure.
     ier : int
@@ -260,33 +255,41 @@ def leastsq(func, x0, args=(), Dfun=None, full_output=0,
     -----
     "leastsq" is a wrapper around MINPACK's lmdif and lmder algorithms.
 
-    From scipy 0.8.0 `leastsq` returns an array of size one instead of a scalar
-    when solving for a single parameter.
+    cov_x is a Jacobian approximation to the Hessian of the least squares
+    objective function.
+    This approximation assumes that the objective function is based on the
+    difference between some observed target data (ydata) and a (non-linear)
+    function of the parameters `f(xdata, params)` ::
+
+           func(params) = ydata - f(xdata, params)
+
+    so that the objective function is ::
+
+           min   sum((ydata - f(xdata, params))**2, axis=0)
+         params
 
     """
-    if not warning :
-        msg = "The warning keyword is deprecated. Use the warnings module."
-        warnings.warn(msg, DeprecationWarning)
-    x0 = array(x0,ndmin=1)
+    x0 = array(x0, ndmin=1)
     n = len(x0)
-    if type(args) != type(()): args = (args,)
-    m = check_func(func,x0,args,n)[0]
-    if n>m:
+    if type(args) != type(()):
+        args = (args,)
+    m = _check_func('leastsq', 'func', func, x0, args, n)[0]
+    if n > m:
         raise TypeError('Improper input: N=%s must not exceed M=%s' % (n,m))
     if Dfun is None:
         if (maxfev == 0):
-            maxfev = 200*(n+1)
-        retval = _minpack._lmdif(func, x0, args, full_output,
-                                 ftol, xtol, gtol,
-                                 maxfev, epsfcn, factor, diag)
+            maxfev = 200*(n + 1)
+        retval = _minpack._lmdif(func, x0, args, full_output, ftol, xtol,
+                gtol, maxfev, epsfcn, factor, diag)
     else:
         if col_deriv:
-            check_func(Dfun,x0,args,n,(n,m))
+            _check_func('leastsq', 'Dfun', Dfun, x0, args, n, (n,m))
         else:
-            check_func(Dfun,x0,args,n,(m,n))
+            _check_func('leastsq', 'Dfun', Dfun, x0, args, n, (m,n))
         if (maxfev == 0):
-            maxfev = 100*(n+1)
-        retval = _minpack._lmder(func,Dfun,x0,args,full_output,col_deriv,ftol,xtol,gtol,maxfev,factor,diag)
+            maxfev = 100*(n + 1)
+        retval = _minpack._lmder(func, Dfun, x0, args, full_output, col_deriv,
+                ftol, xtol, gtol, maxfev, factor, diag)
 
     errors = {0:["Improper input parameters.", TypeError],
               1:["Both actual and predicted relative reductions "
@@ -335,7 +338,7 @@ def leastsq(func, x0, args=(), Dfun=None, full_output=0,
                 cov_x = inv(dot(transpose(R),R))
             except LinAlgError:
                 pass
-        return (retval[0], cov_x) + retval[1:-1] + (mesg,info)
+        return (retval[0], cov_x) + retval[1:-1] + (mesg, info)
     else:
         return (retval[0], info)
 
@@ -419,6 +422,7 @@ def curve_fit(f, xdata, ydata, p0=None, sigma=None, **kw):
     else:
         func = _weighted_general_function
         args += (1.0/asarray(sigma),)
+
     res = leastsq(func, p0, args=args, full_output=1, **kw)
     (popt, pcov, infodict, errmsg, ier) = res
 
@@ -434,132 +438,35 @@ def curve_fit(f, xdata, ydata, p0=None, sigma=None, **kw):
 
     return popt, pcov
 
-def check_gradient(fcn,Dfcn,x0,args=(),col_deriv=0):
+def check_gradient(fcn, Dfcn, x0, args=(), col_deriv=0):
     """Perform a simple check on the gradient for correctness.
 
     """
 
     x = atleast_1d(x0)
     n = len(x)
-    x=x.reshape((n,))
+    x = x.reshape((n,))
     fvec = atleast_1d(fcn(x,*args))
     m = len(fvec)
-    fvec=fvec.reshape((m,))
+    fvec = fvec.reshape((m,))
     ldfjac = m
     fjac = atleast_1d(Dfcn(x,*args))
-    fjac=fjac.reshape((m,n))
+    fjac = fjac.reshape((m,n))
     if col_deriv == 0:
         fjac = transpose(fjac)
 
     xp = zeros((n,), float)
     err = zeros((m,), float)
     fvecp = None
-    _minpack._chkder(m,n,x,fvec,fjac,ldfjac,xp,fvecp,1,err)
+    _minpack._chkder(m, n, x, fvec, fjac, ldfjac, xp, fvecp, 1, err)
 
     fvecp = atleast_1d(fcn(xp,*args))
-    fvecp=fvecp.reshape((m,))
-    _minpack._chkder(m,n,x,fvec,fjac,ldfjac,xp,fvecp,2,err)
+    fvecp = fvecp.reshape((m,))
+    _minpack._chkder(m, n, x, fvec, fjac, ldfjac, xp, fvecp, 2, err)
 
-    good = (product(greater(err,0.5),axis=0))
+    good = (product(greater(err, 0.5), axis=0))
 
-    return (good,err)
-
-
-# Newton-Raphson method
-def newton(func, x0, fprime=None, args=(), tol=1.48e-8, maxiter=50):
-    """Find a zero using the Newton-Raphson or secant method.
-
-    Find a zero of the function `func` given a nearby starting point `x0`.
-    The Newton-Rapheson method is used if the derivative `fprime` of `func`
-    is provided, otherwise the secant method is used.
-
-    Parameters
-    ----------
-    func : function
-        The function whose zero is wanted. It must be a function of a
-        single variable of the form f(x,a,b,c...), where a,b,c... are extra
-        arguments that can be passed in the `args` parameter.
-    x0 : float
-        An initial estimate of the zero that should be somewhere near the
-        actual zero.
-    fprime : {None, function}, optional
-        The derivative of the function when available and convenient. If it
-        is None, then the secant method is used. The default is None.
-    args : tuple, optional
-        Extra arguments to be used in the function call.
-    tol : float, optional
-        The allowable error of the zero value.
-    maxiter : int, optional
-        Maximum number of iterations.
-
-    Returns
-    -------
-    zero : float
-        Estimated location where function is zero.
-
-    See Also
-    --------
-    brentq, brenth, ridder, bisect -- find zeroes in one dimension.
-    fsolve -- find zeroes in n dimensions.
-
-    Notes
-    -----
-    The convergence rate of the Newton-Rapheson method is quadratic while
-    that of the secant method is somewhat less. This means that if the
-    function is  well behaved the actual error in the estimated zero is
-    approximatly the square of the requested tolerance up to roundoff
-    error. However, the stopping criterion used here is the step size and
-    there is no quarantee that a zero has been found. Consequently the
-    result should be verified. Safer algorithms are brentq, brenth, ridder,
-    and bisect, but they all require that the root first be bracketed in an
-    interval where the function changes sign. The brentq algorithm is
-    recommended for general use in one dimemsional problems when such an
-    interval has been found.
-
-    """
-    msg  = "minpack.newton is moving to zeros.newton"
-    warnings.warn(msg, DeprecationWarning)
-
-    if fprime is not None:
-        # Newton-Rapheson method
-        p0 = x0
-        for iter in range(maxiter):
-            myargs = (p0,) + args
-            fval = func(*myargs)
-            fder = fprime(*myargs)
-            if fder == 0:
-                msg = "derivative was zero."
-                warnings.warn(msg, RuntimeWarning)
-                return p0
-            p = p0 - func(*myargs)/fprime(*myargs)
-            if abs(p - p0) < tol:
-                return p
-            p0 = p
-    else:
-        # Secant method
-        p0 = x0
-        if x0 >= 0:
-            p1 = x0*(1 + 1e-4) + 1e-4
-        else:
-            p1 = x0*(1 + 1e-4) - 1e-4
-        q0 = func(*((p0,) + args))
-        q1 = func(*((p1,) + args))
-        for iter in range(maxiter):
-            if q1 == q0:
-                if p1 != p0:
-                    msg = "Tolerance of %s reached" % (p1 - p0)
-                    warnings.warn(msg, RuntimeWarning)
-                return (p1 + p0)/2.0
-            else:
-                p = p1 - q1*(p1 - p0)/(q1 - q0)
-            if abs(p - p1) < tol:
-                return p
-            p0 = p1
-            q0 = q1
-            p1 = p
-            q1 = func(*((p1,) + args))
-    msg = "Failed to converge after %d iterations, value is %s" % (maxiter, p)
-    raise RuntimeError(msg)
+    return (good, err)
 
 
 # Steffensen's Method using Aitken's Del^2 convergence acceleration.
@@ -591,9 +498,9 @@ def fixed_point(func, x0, args=(), xtol=1e-8, maxiter=500):
             p1 = func(p0, *args)
             p2 = func(p1, *args)
             d = p2 - 2.0 * p1 + p0
-            p = where(d == 0, p2, p0 - (p1 - p0)*(p1-p0) / d)
+            p = where(d == 0, p2, p0 - (p1 - p0)*(p1 - p0) / d)
             relerr = where(p0 == 0, p, (p-p0)/p0)
-            if all(relerr < xtol):
+            if all(abs(relerr) < xtol):
                 return p
             p0 = p
     else:
@@ -605,45 +512,13 @@ def fixed_point(func, x0, args=(), xtol=1e-8, maxiter=500):
             if d == 0.0:
                 return p2
             else:
-                p = p0 - (p1 - p0)*(p1-p0) / d
+                p = p0 - (p1 - p0)*(p1 - p0) / d
             if p0 == 0:
                 relerr = p
             else:
-                relerr = (p-p0)/p0
-            if relerr < xtol:
+                relerr = (p - p0)/p0
+            if abs(relerr) < xtol:
                 return p
             p0 = p
     msg = "Failed to converge after %d iterations, value is %s" % (maxiter, p)
     raise RuntimeError(msg)
-
-
-def bisection(func, a, b, args=(), xtol=1e-10, maxiter=400):
-    """Bisection root-finding method.  Given a function and an interval with
-    func(a) * func(b) < 0, find the root between a and b.
-
-    """
-    msg = "minpack.bisection is deprecated, use zeros.bisect instead"
-    warnings.warn(msg, DeprecationWarning)
-
-    i = 1
-    eva = func(a,*args)
-    evb = func(b,*args)
-    if eva*evb >= 0:
-        msg = "Must start with interval where func(a) * func(b) < 0"
-        raise ValueError(msg)
-    while i <= maxiter:
-        dist = (b - a)/2.0
-        p = a + dist
-        if dist < xtol:
-            return p
-        ev = func(p,*args)
-        if ev == 0:
-            return p
-        i += 1
-        if ev*eva > 0:
-            a = p
-            eva = ev
-        else:
-            b = p
-    msg = "Failed to converge after %d iterations, value is %s" % (maxiter, p)
-    raise RuntimeError(msg)
