@@ -1,3 +1,6 @@
+
+import warnings
+
 import _zeros
 from numpy import finfo
 
@@ -97,13 +100,12 @@ def newton(func, x0, fprime=None, args=(), tol=1.48e-8, maxiter=50):
         p0 = 1.0 * x0
         for iter in range(maxiter):
             myargs = (p0,) + args
-            fval = func(*myargs)
             fder = fprime(*myargs)
             if fder == 0:
                 msg = "derivative was zero."
                 warnings.warn(msg, RuntimeWarning)
                 return p0
-            p = p0 - func(*myargs)/fprime(*myargs)
+            p = p0 - func(*myargs) / fder
             if abs(p - p0) < tol:
                 return p
             p0 = p
