@@ -1,6 +1,9 @@
 """Filter design.
 """
 
+import types
+import warnings
+
 import numpy
 from numpy import atleast_1d, poly, polyval, roots, real, asarray, allclose, \
     resize, pi, absolute, logspace, r_, sqrt, tan, log10, arctan, arcsinh, \
@@ -8,8 +11,9 @@ from numpy import atleast_1d, poly, polyval, roots, real, asarray, allclose, \
 from numpy import mintypecode
 from scipy import special, optimize
 from scipy.misc import comb
-import string, types
 
+class BadCoefficients(UserWarning):
+    pass
 
 abs = absolute
 
@@ -29,29 +33,50 @@ def findfreqs(num, den, N):
     w = logspace(lfreq, hfreq, N)
     return w
 
-def freqs(b,a,worN=None,plot=None):
-    """Compute frequency response of analog filter.
+def freqs(b, a, worN=None, plot=None):
+    """
+    Compute frequency response of analog filter.
 
-    Description:
+    Given the numerator (b) and denominator (a) of a filter compute its
+    frequency response::
 
-       Given the numerator (b) and denominator (a) of a filter compute its
-       frequency response.
+             b[0]*(jw)**(nb-1) + b[1]*(jw)**(nb-2) + ... + b[nb-1]
+     H(w) = -------------------------------------------------------
+             a[0]*(jw)**(na-1) + a[1]*(jw)**(na-2) + ... + a[na-1]
 
-               b[0]*(jw)**(nb-1) + b[1]*(jw)**(nb-2) + ... + b[nb-1]
-       H(w) = --------------------------------------------------------
-               a[0]*(jw)**(na-1) + a[1]*(jw)**(na-2) + ... + a[na-1]
+    Parameters
+    ----------
+    b : ndarray
+        Numerator of a linear filter.
+    a : ndarray
+        Denominator of a linear filter.
+    worN : {None, int}, optional
+        If None, then compute at 200 frequencies around the interesting parts
+        of the response curve (determined by pole-zero locations).  If a single
+        integer, the compute at that many frequencies.  Otherwise, compute the
+        response at frequencies given in worN.
+    plot : callable
+        A callable that takes two arguments. If given, the return parameters
+        `w` and `h` are passed to plot. Useful for plotting the frequency
+        response inside `freqz`.
 
-    Inputs:
+    Returns
+    -------
+    w : ndarray
+        The frequencies at which h was computed.
+    h : ndarray
+        The frequency response.
 
-       b, a --- the numerator and denominator of a linear filter.
-       worN --- If None, then compute at 200 frequencies around the interesting
-                parts of the response curve (determined by pole-zero locations).
-                If a single integer, the compute at that many frequencies.
-                Otherwise, compute the response at frequencies given in worN.
-    Outputs: (w,h)
+    See Also
+    --------
+    freqz : Compute the frequency response of a digital filter.
+
+    Notes
+    -----
+    Using Matplotlib's "plot" function as the callable for `plot` produces
+    unexpected results,  this plots the real part of the complex transfer
+    function, not the magnitude.
 
-       w -- The frequencies at which h was computed.
-       h -- The frequency response.
     """
     if worN is None:
         w = findfreqs(b,a,200)
@@ -68,33 +93,70 @@ def freqs(b,a,worN=None,plot=None):
     return w, h
 
 def freqz(b, a=1, worN=None, whole=0, plot=None):
-    """Compute frequency response of a digital filter.
+    """
+    Compute the frequency response of a digital filter.
 
-    Description:
+    Given the numerator ``b`` and denominator ``a`` of a digital filter compute
+    its frequency response::
 
-       Given the numerator (b) and denominator (a) of a digital filter compute
-       its frequency response.
+               jw               -jw            -jmw
+        jw  B(e)    b[0] + b[1]e + .... + b[m]e
+     H(e) = ---- = ------------------------------------
+               jw               -jw            -jnw
+            A(e)    a[0] + a[1]e + .... + a[n]e
 
-                  jw               -jw            -jmw
-           jw  B(e)    b[0] + b[1]e + .... + b[m]e
-        H(e) = ---- = ------------------------------------
-                  jw               -jw            -jnw
-               A(e)    a[0] + a[2]e + .... + a[n]e
+    Parameters
+    ----------
+    b : ndarray
+        numerator of a linear filter
+    a : ndarray
+        denominator of a linear filter
+    worN : {None, int}, optional
+        If None, then compute at 512 frequencies around the unit circle.
+        If a single integer, the compute at that many frequencies.
+        Otherwise, compute the response at frequencies given in worN
+    whole : bool, optional
+        Normally, frequencies are computed from 0 to pi (upper-half of
+        unit-circle.  If whole is False, compute frequencies from 0 to 2*pi.
+    plot : callable
+        A callable that takes two arguments. If given, the return parameters
+        `w` and `h` are passed to plot. Useful for plotting the frequency
+        response inside `freqz`.
 
-    Inputs:
+    Returns
+    -------
+    w : ndarray
+        The frequencies at which h was computed.
+    h : ndarray
+        The frequency response.
 
-       b, a --- the numerator and denominator of a linear filter.
-       worN --- If None, then compute at 512 frequencies around the unit circle.
-                If a single integer, the compute at that many frequencies.
-                Otherwise, compute the response at frequencies given in worN
-       whole -- Normally, frequencies are computed from 0 to pi (upper-half of
-                unit-circle.  If whole is non-zero compute frequencies from 0
-                to 2*pi.
+    Notes
+    -----
+    Using Matplotlib's "plot" function as the callable for `plot` produces
+    unexpected results,  this plots the real part of the complex transfer
+    function, not the magnitude.
 
-    Outputs: (w,h)
+    Examples
+    --------
+    >>> b = firwin(80, 0.5, window=('kaiser', 8))
+    >>> h, w = freqz(b)
 
-       w -- The frequencies at which h was computed.
-       h -- The frequency response.
+    >>> import matplotlib.pyplot as plt
+    >>> fig = plt.figure()
+    >>> plt.title('Digital filter frequency response')
+    >>> ax1 = fig.add_subplot(111)
+
+    >>> plt.semilogy(h, np.abs(w), 'b')
+    >>> plt.ylabel('Amplitude (dB)', color='b')
+    >>> plt.xlabel('Frequency (rad/sample)')
+    >>> plt.grid()
+    >>> plt.legend()
+
+    >>> ax2 = ax1.twinx()
+    >>> angles = np.unwrap(np.angle(w))
+    >>> plt.plot(h, angles, 'g')
+    >>> plt.ylabel('Angle (radians)', color='g')
+    >>> plt.show()
 
     """
     b, a = map(atleast_1d, (b,a))
@@ -117,9 +179,28 @@ def freqz(b, a=1, worN=None, whole=0, plot=None):
         plot(w, h)
     return w, h
 
-def tf2zpk(b,a):
+def tf2zpk(b, a):
     """Return zero, pole, gain (z,p,k) representation from a numerator,
     denominator representation of a linear filter.
+
+    Parameters
+    ----------
+    b : ndarray
+        numerator polynomial.
+    a : ndarray
+        numerator and denominator polynomials.
+
+    Returns
+    -------
+    z : ndarray
+        Zeros of the transfer function.
+    p : ndarray
+        Poles of the transfer function.
+    k : float
+        System gain.
+
+    If some values of b are too close to 0, they are removed. In that case, a
+    BadCoefficients warning is emitted.
     """
     b,a = normalize(b,a)
     b = (b+0.0) / a[0]
@@ -130,18 +211,26 @@ def tf2zpk(b,a):
     p = roots(a)
     return z, p, k
 
-def zpk2tf(z,p,k):
+def zpk2tf(z, p, k):
     """Return polynomial transfer function representation from zeros
     and poles
 
-    Inputs:
+    Parameters
+    ----------
+    z : ndarray
+        Zeros of the transfer function.
+    p : ndarray
+        Poles of the transfer function.
+    k : float
+        System gain.
 
-      z, p --- sequences representing the zeros and poles.
-      k --- system gain.
+    Returns
+    -------
+    b : ndarray
+        Numerator polynomial.
+    a : ndarray
+        Numerator and denominator polynomials.
 
-    Outputs: (b,a)
-
-      b, a --- numerator and denominator polynomials.
     """
     z = atleast_1d(z)
     k = atleast_1d(k)
@@ -157,29 +246,35 @@ def zpk2tf(z,p,k):
     a = poly(p)
     return b, a
 
-def normalize(b,a):
+def normalize(b, a):
     """Normalize polynomial representation of a transfer function.
+
+    If values of b are too close to 0, they are removed. In that case, a
+    BadCoefficients warning is emitted.
     """
     b,a = map(atleast_1d,(b,a))
     if len(a.shape) != 1:
-        raise ValueError, "Denominator polynomial must be rank-1 array."
+        raise ValueError("Denominator polynomial must be rank-1 array.")
     if len(b.shape) > 2:
-        raise ValueError, "Numerator polynomial must be rank-1 or rank-2 array."
+        raise ValueError("Numerator polynomial must be rank-1 or rank-2 array.")
     if len(b.shape) == 1:
         b = asarray([b],b.dtype.char)
     while a[0] == 0.0 and len(a) > 1:
         a = a[1:]
-    while allclose(b[:,0], 0, rtol=1e-14) and (b.shape[-1] > 1):
-        b = b[:,1:]
-    if b.shape[0] == 1:
-        b = b[0]
     outb = b * (1.0) / a[0]
     outa = a * (1.0) / a[0]
+    if allclose(outb[:,0], 0, rtol=1e-14):
+        warnings.warn("Badly conditioned filter coefficients (numerator): the "
+                      "results may be meaningless", BadCoefficients)
+        while allclose(outb[:,0], 0, rtol=1e-14) and (outb.shape[-1] > 1):
+            outb = outb[:,1:]
+    if outb.shape[0] == 1:
+        outb = outb[0]
     return outb, outa
 
 
-def lp2lp(b,a,wo=1.0):
-    """Return a low-pass filter with cuttoff frequency wo
+def lp2lp(b, a, wo=1.0):
+    """Return a low-pass filter with cutoff frequency `wo`
     from a low-pass filter prototype with unity cutoff frequency.
     """
     a,b = map(atleast_1d,(a,b))
@@ -197,8 +292,8 @@ def lp2lp(b,a,wo=1.0):
     a = a * pwo[start1]/pwo[start1:]
     return normalize(b, a)
 
-def lp2hp(b,a,wo=1.0):
-    """Return a high-pass filter with cuttoff frequency wo
+def lp2hp(b, a, wo=1.0):
+    """Return a high-pass filter with cutoff frequency `wo`
     from a low-pass filter prototype with unity cutoff frequency.
     """
     a,b = map(atleast_1d,(a,b))
@@ -225,8 +320,8 @@ def lp2hp(b,a,wo=1.0):
 
     return normalize(outb, outa)
 
-def lp2bp(b,a,wo=1.0, bw=1.0):
-    """Return a band-pass filter with center frequency wo and bandwidth bw
+def lp2bp(b, a, wo=1.0, bw=1.0):
+    """Return a band-pass filter with center frequency `wo` and bandwidth `bw`
     from a low-pass filter prototype with unity cutoff frequency.
     """
     a,b = map(atleast_1d,(a,b))
@@ -256,8 +351,8 @@ def lp2bp(b,a,wo=1.0, bw=1.0):
 
     return normalize(bprime, aprime)
 
-def lp2bs(b,a,wo=1,bw=1):
-    """Return a band-stop filter with center frequency wo and bandwidth bw
+def lp2bs(b, a, wo=1, bw=1):
+    """Return a band-stop filter with center frequency `wo` and bandwidth `bw`
     from a low-pass filter prototype with unity cutoff frequency.
     """
     a,b = map(atleast_1d,(a,b))
@@ -287,10 +382,10 @@ def lp2bs(b,a,wo=1,bw=1):
 
     return normalize(bprime, aprime)
 
-def bilinear(b,a,fs=1.0):
+def bilinear(b, a, fs=1.0):
     """Return a digital filter from an analog filter using the bilinear transform.
 
-    The bilinear transform substitutes (z-1) / (z+1) for s
+    The bilinear transform substitutes ``(z-1) / (z+1``) for ``s``.
     """
     fs =float(fs)
     a,b = map(atleast_1d,(a,b))
@@ -324,44 +419,57 @@ def bilinear(b,a,fs=1.0):
 def iirdesign(wp, ws, gpass, gstop, analog=0, ftype='ellip', output='ba'):
     """Complete IIR digital and analog filter design.
 
-    Description:
+    Given passband and stopband frequencies and gains construct an analog or
+    digital IIR filter of minimum order for a given basic type.  Return the
+    output in numerator, denominator ('ba') or pole-zero ('zpk') form.
 
-      Given passband and stopband frequencies and gains construct an analog or
-      digital IIR filter of minimum order for a given basic type.  Return the
-      output in numerator, denominator ('ba') or pole-zero ('zpk') form.
+    Parameters
+    ----------
+    wp, ws : float
+        Passband and stopband edge frequencies, normalized from 0 to 1 (1
+        corresponds to pi radians / sample).  For example:
 
-    Inputs:
+            - Lowpass:   wp = 0.2,          ws = 0.3
+            - Highpass:  wp = 0.3,          ws = 0.2
+            - Bandpass:  wp = [0.2, 0.5],   ws = [0.1, 0.6]
+            - Bandstop:  wp = [0.1, 0.6],   ws = [0.2, 0.5]
 
-      wp, ws -- Passband and stopband edge frequencies, normalized from 0
-                to 1 (1 corresponds to pi radians / sample).  For example:
-                   Lowpass:   wp = 0.2,          ws = 0.3
-                   Highpass:  wp = 0.3,          ws = 0.2
-                   Bandpass:  wp = [0.2, 0.5],   ws = [0.1, 0.6]
-                   Bandstop:  wp = [0.1, 0.6],   ws = [0.2, 0.5]
-      gpass -- The maximum loss in the passband (dB).
-      gstop -- The minimum attenuation in the stopband (dB).
-      analog -- Non-zero to design an analog filter (in this case wp and
-                ws are in radians / second).
-      ftype -- The type of iir filter to design:
-                 elliptic    : 'ellip'
-                 Butterworth : 'butter',
-                 Chebyshev I : 'cheby1',
-                 Chebyshev II: 'cheby2',
-                 Bessel :      'bessel'
-      output -- Type of output:  numerator/denominator ('ba') or pole-zero ('zpk')
+    gpass : float
+        The maximum loss in the passband (dB).
+    gstop : float
+        The minimum attenuation in the stopband (dB).
+    analog : int, optional
+        Non-zero to design an analog filter (in this case `wp` and `ws` are in
+        radians / second).
+    ftype : str, optional
+        The type of IIR filter to design:
 
-    Outputs: (b,a) or (z,p,k)
+            - elliptic    : 'ellip'
+            - Butterworth : 'butter',
+            - Chebyshev I : 'cheby1',
+            - Chebyshev II: 'cheby2',
+            - Bessel :      'bessel'
+
+    output : ['ba', 'zpk'], optional
+        Type of output:  numerator/denominator ('ba') or pole-zero ('zpk').
+        Default is 'ba'.
+
+    Returns
+    -------
+    b, a :
+        Numerator and denominator of the IIR filter. Only returned if
+        ``output='ba'``.
+    z, p, k : Zeros, poles, and gain of the IIR filter. Only returned if
+    ``output='zpk'``.
 
-      b,a -- Numerator and denominator of the iir filter.
-      z,p,k -- Zeros, poles, and gain of the iir filter.
     """
 
     try:
         ordfunc = filter_dict[ftype][1]
     except KeyError:
-        raise ValueError, "Invalid IIR filter type."
+        raise ValueError("Invalid IIR filter type: %s" % ftype)
     except IndexError:
-        raise ValueError, "%s does not have order selection use iirfilter function." % ftype
+        raise ValueError("%s does not have order selection use iirfilter function." % ftype)
 
     wp = atleast_1d(wp)
     ws = atleast_1d(ws)
@@ -379,41 +487,59 @@ def iirdesign(wp, ws, gpass, gstop, analog=0, ftype='ellip', output='ba'):
 def iirfilter(N, Wn, rp=None, rs=None, btype='band', analog=0, ftype='butter', output='ba'):
     """IIR digital and analog filter design given order and critical points.
 
-    Description:
+    Design an Nth order lowpass digital or analog filter and return the filter
+    coefficients in (B,A) (numerator, denominator) or (Z,P,K) form.
 
-      Design an Nth order lowpass digital or analog filter and return the filter
-      coefficients in (B,A) (numerator, denominator) or (Z,P,K) form.
+    Parameters
+    ----------
+    N : int
+        The order of the filter.
+    Wn : array_like
+        A scalar or length-2 sequence giving the critical frequencies.
+    rp : float, optional
+        For Chebyshev and elliptic filters provides the maximum ripple
+        in the passband.
+    rs : float, optional
+        For chebyshev and elliptic filters provides the minimum attenuation in
+        the stop band.
+    btype : str, optional
+        The type of filter (lowpass, highpass, bandpass, bandstop).
+        Default is bandpass.
+    analog : int, optional
+        Non-zero to return an analog filter, otherwise a digital filter is
+        returned.
+    ftype : str, optional
+        The type of IIR filter to design:
 
-    Inputs:
+            - elliptic    : 'ellip'
+            - Butterworth : 'butter',
+            - Chebyshev I : 'cheby1',
+            - Chebyshev II: 'cheby2',
+            - Bessel :      'bessel'
 
-      N -- the order of the filter.
-      Wn -- a scalar or length-2 sequence giving the critical frequencies.
-      rp, rs -- For chebyshev and elliptic filters provides the maximum ripple
-                in the passband and the minimum attenuation in the stop band.
-      btype -- the type of filter (lowpass, highpass, bandpass, or bandstop).
-      analog -- non-zero to return an analog filter, otherwise
-                a digital filter is returned.
-      ftype -- the type of IIR filter (Butterworth, Cauer (Elliptic),
-               Bessel, Chebyshev1, Chebyshev2)
-      output -- 'ba' for (b,a) output, 'zpk' for (z,p,k) output.
+    output : ['ba', 'zpk'], optional
+        Type of output:  numerator/denominator ('ba') or pole-zero ('zpk').
+        Default is 'ba'.
 
-    SEE ALSO butterord, cheb1ord, cheb2ord, ellipord
+    See Also
+    --------
+    butterord, cheb1ord, cheb2ord, ellipord
     """
 
-    ftype, btype, output = map(string.lower, (ftype, btype, output))
+    ftype, btype, output = [x.lower() for x in (ftype, btype, output)]
     Wn = asarray(Wn)
     try:
         btype = band_dict[btype]
     except KeyError:
-        raise ValueError, "%s is an invalid bandtype for filter." % btype
+        raise ValueError("%s is an invalid bandtype for filter." % btype)
 
     try:
         typefunc = filter_dict[ftype][0]
     except KeyError:
-        raise ValueError, "%s is not a valid basic iir filter." % ftype
+        raise ValueError("%s is not a valid basic iir filter." % ftype)
 
     if output not in ['ba', 'zpk']:
-        raise ValueError, "%s is not a valid output form." % output
+        raise ValueError("%s is not a valid output form." % output)
 
     #pre-warp frequencies for digital filter design
     if not analog:
@@ -434,15 +560,15 @@ def iirfilter(N, Wn, rp=None, rs=None, btype='band', analog=0, ftype='butter', o
         z, p, k = typefunc(N)
     elif typefunc == cheb1ap:
         if rp is None:
-            raise ValueError, "passband ripple (rp) must be provided to design a Chebyshev I filter."
+            raise ValueError("passband ripple (rp) must be provided to design a Chebyshev I filter.")
         z, p, k = typefunc(N, rp)
     elif typefunc == cheb2ap:
         if rs is None:
-            raise ValueError, "stopband atteunatuion (rs) must be provided to design an Chebyshev II filter."
+            raise ValueError("stopband atteunatuion (rs) must be provided to design an Chebyshev II filter.")
         z, p, k = typefunc(N, rs)
     else:  # Elliptic filters
         if rs is None or rp is None:
-            raise ValueError, "Both rp and rs must be provided to design an elliptic filter."
+            raise ValueError("Both rp and rs must be provided to design an elliptic filter.")
         z, p, k = typefunc(N, rp, rs)
 
     b, a = zpk2tf(z,p,k)
@@ -472,58 +598,56 @@ def iirfilter(N, Wn, rp=None, rs=None, btype='band', analog=0, ftype='butter', o
 def butter(N, Wn, btype='low', analog=0, output='ba'):
     """Butterworth digital and analog filter design.
 
-    Description:
+    Design an Nth order lowpass digital or analog Butterworth filter and return
+    the filter coefficients in (B,A) or (Z,P,K) form.
 
-      Design an Nth order lowpass digital or analog Butterworth filter
-      and return the filter coefficients in (B,A) or (Z,P,K) form.
-
-    See also buttord.
+    See also
+    --------
+    buttord.
     """
     return iirfilter(N, Wn, btype=btype, analog=analog, output=output, ftype='butter')
 
 def cheby1(N, rp, Wn, btype='low', analog=0, output='ba'):
     """Chebyshev type I digital and analog filter design.
 
-    Description:
+    Design an Nth order lowpass digital or analog Chebyshev type I filter and
+    return the filter coefficients in (B,A) or (Z,P,K) form.
 
-      Design an Nth order lowpass digital or analog Chebyshev type I filter
-      and return the filter coefficients in (B,A) or (Z,P,K) form.
-
-    See also cheb1ord.
+    See also
+    --------
+    cheb1ord.
     """
     return iirfilter(N, Wn, rp=rp, btype=btype, analog=analog, output=output, ftype='cheby1')
 
 def cheby2(N, rs, Wn, btype='low', analog=0, output='ba'):
     """Chebyshev type I digital and analog filter design.
 
-    Description:
+    Design an Nth order lowpass digital or analog Chebyshev type I filter and
+    return the filter coefficients in (B,A) or (Z,P,K) form.
 
-      Design an Nth order lowpass digital or analog Chebyshev type I filter
-      and return the filter coefficients in (B,A) or (Z,P,K) form.
-
-    See also cheb2ord.
+    See also
+    --------
+    cheb2ord.
     """
     return iirfilter(N, Wn, rs=rs, btype=btype, analog=analog, output=output, ftype='cheby2')
 
 def ellip(N, rp, rs, Wn, btype='low', analog=0, output='ba'):
     """Elliptic (Cauer) digital and analog filter design.
 
-    Description:
+    Design an Nth order lowpass digital or analog elliptic filter and return
+    the filter coefficients in (B,A) or (Z,P,K) form.
 
-      Design an Nth order lowpass digital or analog elliptic filter
-      and return the filter coefficients in (B,A) or (Z,P,K) form.
-
-    See also ellipord.
+    See also
+    --------
+    ellipord.
     """
     return iirfilter(N, Wn, rs=rs, rp=rp, btype=btype, analog=analog, output=output, ftype='elliptic')
 
 def bessel(N, Wn, btype='low', analog=0, output='ba'):
     """Bessel digital and analog filter design.
 
-    Description:
-
-      Design an Nth order lowpass digital or analog Bessel filter
-      and return the filter coefficients in (B,A) or (Z,P,K) form.
+    Design an Nth order lowpass digital or analog Bessel filter and return the
+    filter coefficients in (B,A) or (Z,P,K) form.
 
     """
     return iirfilter(N, Wn, btype=btype, analog=analog, output=output, ftype='bessel')
@@ -537,25 +661,31 @@ def yulewalk():
 
 
 def band_stop_obj(wp, ind, passb, stopb, gpass, gstop, type):
-    """Band Stop Objective Function for order minimization
+    """Band Stop Objective Function for order minimization.
 
-    Description:
+    Returns the non-integer order for an analog band stop filter.
 
-      Returns the non-integer order for an analog band stop filter.
+    Parameters
+    ----------
+    wp :
+        Edge of passband `passb`.
+    ind : int
+        Index specifying which `passb` edge to vary (0 or 1).
+    passb : array_like
+        Two element sequence of fixed passband edges.
+    stopb : array_like
+        Two element sequence of fixed stopband edges.
+    gstop : float
+        Amount of attenuation in stopband in dB.
+    gpass : float
+        Amount of ripple in the passband in dB.
+    type : ['butter', 'cheby', 'ellip']
+        Type of filter.
 
-    Inputs:
-
-      wp -- passb edge
-      ind -- index specifying which passb edge to vary (0 or 1).
-      passb -- two element vector of fixed passband edges.
-      stopb -- two element vector of fixed stopband edges.
-      gstop -- amount in dB of attenuation in stopband.
-      gpass -- amount in dB of ripple in the passband.
-      type -- 'butter', 'cheby', or 'ellip':
-
-    Outputs: (n,)
-
-      n -- filter order (possibly non-integer)
+    Returns
+    -------
+    n : scalar
+        Filter order (possibly non-integer).
     """
 
     passbC = passb.copy()
@@ -580,36 +710,42 @@ def band_stop_obj(wp, ind, passb, stopb, gpass, gstop, type):
         d1 = special.ellipk([arg1**2, 1-arg1**2])
         n = (d0[0]*d1[1] / (d0[1]*d1[0]))
     else:
-        raise ValueError, "Incorrect type: ", type
+        raise ValueError("Incorrect type: %s" % type)
     return n
 
 def buttord(wp, ws, gpass, gstop, analog=0):
     """Butterworth filter order selection.
 
-    Description:
+    Return the order of the lowest order digital Butterworth filter that loses
+    no more than `gpass` dB in the passband and has at least `gstop` dB
+    attenuation in the stopband.
 
-      Return the order of the lowest order digital Butterworth filter that
-      loses no more than gpass dB in the passband and has at least gstop dB
-      attenuation in the stopband.
+    Parameters
+    ----------
+    wp, ws : float
+        Passband and stopband edge frequencies, normalized from 0 to 1 (1
+        corresponds to pi radians / sample).  For example:
 
-    Inputs:
+            - Lowpass:   wp = 0.2,          ws = 0.3
+            - Highpass:  wp = 0.3,          ws = 0.2
+            - Bandpass:  wp = [0.2, 0.5],   ws = [0.1, 0.6]
+            - Bandstop:  wp = [0.1, 0.6],   ws = [0.2, 0.5]
 
-      wp, ws -- Passband and stopband edge frequencies, normalized from 0
-                to 1 (1 corresponds to pi radians / sample).  For example:
-                   Lowpass:   wp = 0.2,          ws = 0.3
-                   Highpass:  wp = 0.3,          ws = 0.2
-                   Bandpass:  wp = [0.2, 0.5],   ws = [0.1, 0.6]
-                   Bandstop:  wp = [0.1, 0.6],   ws = [0.2, 0.5]
-      gpass -- The maximum loss in the passband (dB).
-      gstop -- The minimum attenuation in the stopband (dB).
-      analog -- Non-zero to design an analog filter (in this case wp and
-                ws are in radians / second).
+    gpass : float
+        The maximum loss in the passband (dB).
+    gstop : float
+        The minimum attenuation in the stopband (dB).
+    analog : int, optional
+        Non-zero to design an analog filter (in this case `wp` and `ws` are in
+        radians / second).
 
-    Outputs: (ord, Wn)
-
-      ord -- The lowest order for a Butterworth filter which meets specs.
-      Wn -- The Butterworth natural frequency (i.e. the "3dB frequency").
-            Should be used with scipy.signal.butter to give filter results.
+    Returns
+    -------
+    ord : int
+        The lowest order for a Butterworth filter which meets specs.
+    wn : ndarray or float
+        The Butterworth natural frequency (i.e. the "3dB frequency").  Should
+        be used with `butter` to give filter results.
 
     """
 
@@ -680,7 +816,7 @@ def buttord(wp, ws, gpass, gstop, analog=0):
                                               passb[0]*passb[1])
         WN = numpy.sort(abs(WN))
     else:
-        raise ValueError, "Bad type."
+        raise ValueError("Bad type: %s" % filter_type)
 
     if not analog:
         wn = (2.0/pi)*arctan(WN)
@@ -694,30 +830,36 @@ def buttord(wp, ws, gpass, gstop, analog=0):
 def cheb1ord(wp, ws, gpass, gstop, analog=0):
     """Chebyshev type I filter order selection.
 
-    Description:
+    Return the order of the lowest order digital Chebyshev Type I filter that
+    loses no more than `gpass` dB in the passband and has at least `gstop` dB
+    attenuation in the stopband.
 
-      Return the order of the lowest order digital Chebyshev Type I filter
-      that loses no more than gpass dB in the passband and has at least gstop dB
-      attenuation in the stopband.
+    Parameters
+    ----------
+    wp, ws : float
+        Passband and stopband edge frequencies, normalized from 0 to 1 (1
+        corresponds to pi radians / sample).  For example:
 
-    Inputs:
+            - Lowpass:   wp = 0.2,          ws = 0.3
+            - Highpass:  wp = 0.3,          ws = 0.2
+            - Bandpass:  wp = [0.2, 0.5],   ws = [0.1, 0.6]
+            - Bandstop:  wp = [0.1, 0.6],   ws = [0.2, 0.5]
 
-      wp, ws -- Passband and stopband edge frequencies, normalized from 0
-                to 1 (1 corresponds to pi radians / sample).  For example:
-                   Lowpass:   wp = 0.2,          ws = 0.3
-                   Highpass:  wp = 0.3,          ws = 0.2
-                   Bandpass:  wp = [0.2, 0.5],   ws = [0.1, 0.6]
-                   Bandstop:  wp = [0.1, 0.6],   ws = [0.2, 0.5]
-      gpass -- The maximum loss in the passband (dB).
-      gstop -- The minimum attenuation in the stopband (dB).
-      analog -- Non-zero to design an analog filter (in this case wp and
-                ws are in radians / second).
+    gpass : float
+        The maximum loss in the passband (dB).
+    gstop : float
+        The minimum attenuation in the stopband (dB).
+    analog : int, optional
+        Non-zero to design an analog filter (in this case `wp` and `ws` are in
+        radians / second).
 
-    Outputs: (ord, Wn)
-
-      ord -- The lowest order for a Chebyshev type I filter that meets specs.
-      Wn -- The Chebyshev natural frequency (the "3dB frequency") for
-            use with scipy.signal.cheby1 to give filter results.
+    Returns
+    -------
+    ord : int
+        The lowest order for a Chebyshev type I filter that meets specs.
+    wn : ndarray or float
+        The Chebyshev natural frequency (the "3dB frequency") for use with
+        `cheby1` to give filter results.
 
     """
     wp = atleast_1d(wp)
@@ -777,24 +919,32 @@ def cheb2ord(wp, ws, gpass, gstop, analog=0):
       that loses no more than gpass dB in the passband and has at least gstop dB
       attenuation in the stopband.
 
-    Inputs:
+    Parameters
+    ----------
+    wp, ws : float
+        Passband and stopband edge frequencies, normalized from 0 to 1 (1
+        corresponds to pi radians / sample).  For example:
 
-      wp, ws -- Passband and stopband edge frequencies, normalized from 0
-                to 1 (1 corresponds to pi radians / sample).  For example:
-                   Lowpass:   wp = 0.2,          ws = 0.3
-                   Highpass:  wp = 0.3,          ws = 0.2
-                   Bandpass:  wp = [0.2, 0.5],   ws = [0.1, 0.6]
-                   Bandstop:  wp = [0.1, 0.6],   ws = [0.2, 0.5]
-      gpass -- The maximum loss in the passband (dB).
-      gstop -- The minimum attenuation in the stopband (dB).
-      analog -- Non-zero to design an analog filter (in this case wp and
-                ws are in radians / second).
+            - Lowpass:   wp = 0.2,          ws = 0.3
+            - Highpass:  wp = 0.3,          ws = 0.2
+            - Bandpass:  wp = [0.2, 0.5],   ws = [0.1, 0.6]
+            - Bandstop:  wp = [0.1, 0.6],   ws = [0.2, 0.5]
 
-    Outputs: (ord, Wn)
+    gpass : float
+        The maximum loss in the passband (dB).
+    gstop : float
+        The minimum attenuation in the stopband (dB).
+    analog : int, optional
+        Non-zero to design an analog filter (in this case `wp` and `ws` are in
+        radians / second).
 
-      ord -- The lowest order for a Chebyshev type II filter that meets specs.
-      Wn -- The Chebyshev natural frequency for
-            use with scipy.signal.cheby2 to give the filter.
+    Returns
+    -------
+    ord : int
+        The lowest order for a Chebyshev type II filter that meets specs.
+    wn : ndarray or float
+        The Chebyshev natural frequency (the "3dB frequency") for use with
+        `cheby2` to give filter results.
 
     """
     wp = atleast_1d(wp)
@@ -872,30 +1022,36 @@ def cheb2ord(wp, ws, gpass, gstop, analog=0):
 def ellipord(wp, ws, gpass, gstop, analog=0):
     """Elliptic (Cauer) filter order selection.
 
-    Description:
+    Return the order of the lowest order digital elliptic filter that loses no
+    more than gpass dB in the passband and has at least gstop dB attenuation in
+    the stopband.
 
-      Return the order of the lowest order digital elliptic filter
-      that loses no more than gpass dB in the passband and has at least
-      gstop dB attenuation in the stopband.
+    Parameters
+    ----------
+    wp, ws : float
+        Passband and stopband edge frequencies, normalized from 0 to 1 (1
+        corresponds to pi radians / sample).  For example:
 
-    Inputs:
+            - Lowpass:   wp = 0.2,          ws = 0.3
+            - Highpass:  wp = 0.3,          ws = 0.2
+            - Bandpass:  wp = [0.2, 0.5],   ws = [0.1, 0.6]
+            - Bandstop:  wp = [0.1, 0.6],   ws = [0.2, 0.5]
 
-      wp, ws -- Passband and stopband edge frequencies, normalized from 0
-                to 1 (1 corresponds to pi radians / sample).  For example:
-                   Lowpass:   wp = 0.2,          ws = 0.3
-                   Highpass:  wp = 0.3,          ws = 0.2
-                   Bandpass:  wp = [0.2, 0.5],   ws = [0.1, 0.6]
-                   Bandstop:  wp = [0.1, 0.6],   ws = [0.2, 0.5]
-      gpass -- The maximum loss in the passband (dB).
-      gstop -- The minimum attenuation in the stopband (dB).
-      analog -- Non-zero to design an analog filter (in this case wp and
-                ws are in radians / second).
+    gpass : float
+        The maximum loss in the passband (dB).
+    gstop : float
+        The minimum attenuation in the stopband (dB).
+    analog : int, optional
+        Non-zero to design an analog filter (in this case `wp` and `ws` are in
+        radians / second).
 
-    Outputs: (ord, Wn)
-
-      ord -- The lowest order for an Elliptic (Cauer) filter that meets specs.
-      Wn  -- The natural frequency for use with scipy.signal.ellip
-               to give the filter.
+    Returns
+    ------
+    ord : int
+        The lowest order for  an Elliptic (Cauer) filter that meets specs.
+    wn : ndarray or float
+        The Chebyshev natural frequency (the "3dB frequency") for use with
+        `ellip` to give filter results.-
 
     """
     wp = atleast_1d(wp)
@@ -958,10 +1114,9 @@ def buttap(N):
     k = 1
     return z, p, k
 
-def cheb1ap(N,rp):
-    """Return (z,p,k) zero, pole, gain for Nth order Chebyshev type I
-    lowpass analog filter prototype with rp decibels of ripple
-    in the passband.
+def cheb1ap(N, rp):
+    """Return (z,p,k) zero, pole, gain for Nth order Chebyshev type I lowpass
+    analog filter prototype with `rp` decibels of ripple in the passband.
     """
     z = []
     eps = numpy.sqrt(10**(0.1*rp)-1.0)
@@ -975,10 +1130,9 @@ def cheb1ap(N,rp):
     return z, p, k
     pass
 
-def cheb2ap(N,rs):
-    """Return (z,p,k) zero, pole, gain for Nth order Chebyshev type II
-    lowpass analog filter prototype with rs decibels of ripple
-    in the stopband.
+def cheb2ap(N, rs):
+    """Return (z,p,k) zero, pole, gain for Nth order Chebyshev type II lowpass
+    analog filter prototype with `rs` decibels of ripple in the stopband.
     """
     de = 1.0/sqrt(10**(0.1*rs)-1)
     mu = arcsinh(1.0/de)/N
@@ -1020,13 +1174,16 @@ def kratio(m, k_ratio):
         r = 1e20
     return abs(r)
 
-def ellipap(N,rp,rs):
+def ellipap(N, rp, rs):
     """Return (z,p,k) zeros, poles, and gain of an Nth order normalized
-    prototype elliptic analog lowpass filter with rp decibels of ripple
-    in the passband and a stopband rs decibels down.
+    prototype elliptic analog lowpass filter with `rp` decibels of ripple in
+    the passband and a stopband `rs` decibels down.
+
+    References
+    ----------
+    Lutova, Tosic, and Evans, "Filter Design for Signal Processing", Chapters 5
+    and 12.
 
-    See Chapter 12 and Chapter 5 of "Filter Design for Signal Processing",
-    by Lutova, Tosic, and Evans.  This is
     """
     if N == 1:
         p = -sqrt(1.0/(10**(0.1*rp)-1.0))
@@ -1038,7 +1195,7 @@ def ellipap(N,rp,rs):
     ck1 = eps / numpy.sqrt(10**(0.1*rs)-1)
     ck1p = numpy.sqrt(1-ck1*ck1)
     if ck1p == 1:
-        raise ValueError, "Cannot design a filter with given rp and rs specifications."
+        raise ValueError("Cannot design a filter with given rp and rs specifications.")
 
     wp = 1
     val = special.ellipk([ck1*ck1,ck1p*ck1p])
@@ -1087,8 +1244,8 @@ def ellipap(N,rp,rs):
 
 
 def besselap(N):
-    """Return (z,p,k) zero, pole, gain for analog prototype of an Nth
-    order Bessel filter."""
+    """Return (z,p,k) zero, pole, gain for analog prototype of an Nth order
+    Bessel filter."""
     z = []
     k = 1
     if N == 0:
@@ -1444,7 +1601,7 @@ def besselap(N):
              -.2373280669322028974199184-1.211476658382565356579418*1j,
              -.2373280669322028974199184+1.211476658382565356579418*1j]
     else:
-        raise ValueError, "Bessel Filter not supported for order %d" % N
+        raise ValueError("Bessel Filter not supported for order %d" % N)
 
     return z, p, k
 
@@ -1478,67 +1635,4 @@ band_dict = {'band':'bandpass',
              'h' : 'highpass'
              }
 
-def kaiserord(ripple, width):
-    """Design a Kaiser window to limit ripple and width of transition region.
-
-    Inputs:
-
-      ripple -- positive number specifying maximum ripple in passband (dB)
-                  and minimum ripple in stopband
-      width  -- width of transition region (normalized so that 1 corresponds
-                  to pi radians / sample)
-
-    Outputs:
-
-      N, beta -- the order and beta parameter for the kaiser window.
-
-                 signal.kaiser(N,beta,sym=0) returns the window as does
-                 signal.get_window(beta,N)
-                 signal.get_window(('kaiser',beta),N)
-
-    Uses the empirical equations discovered by Kaiser.
-
-    Oppenheim, Schafer, "Discrete-Time Signal Processing,", p.475-476.
-    """
-    A = abs(ripple)  # in case somebody is confused as to what's meant
-    if (A>50):
-        beta = 0.1102*(A-8.7)
-    elif (A>21):
-        beta = 0.5842*(A-21)**0.4 + 0.07886*(A-21)
-    else:
-        beta = 0.0
-    N = (A-8)/2.285/(pi*width)
-    return ceil(N), beta
-
-def firwin(N, cutoff, width=None, window='hamming'):
-    """FIR Filter Design using windowed ideal filter method.
-
-    Inputs:
-
-      N      -- order of filter (number of taps)
-      cutoff -- cutoff frequency of filter (normalized so that 1 corresponds to
-                  Nyquist or pi radians / sample)
-
-      width  -- if width is not None, then assume it is the approximate width of
-                  the transition region (normalized so that 1 corresonds to pi)
-                  for use in kaiser FIR filter design.
-      window -- desired window to use.
-
-    Outputs:
-
-      h      -- coefficients of length N fir filter.
-    """
-
-    from signaltools import get_window
-    if isinstance(width,float):
-        A = 2.285*N*width + 8
-        if (A < 21): beta = 0.0
-        elif (A <= 50): beta = 0.5842*(A-21)**0.4 + 0.07886*(A-21)
-        else: beta = 0.1102*(A-8.7)
-        window=('kaiser',beta)
-
-    win = get_window(window,N,fftbins=1)
-    alpha = N//2
-    m = numpy.arange(0,N)
-    h = win*special.sinc(cutoff*(m-alpha))
-    return h / numpy.sum(h,axis=0)
+warnings.simplefilter("always", BadCoefficients)
