@@ -18,10 +18,12 @@ import warnings
 import numpy as np
 from numpy import arange, zeros, array, dot, matrix, asmatrix, asarray, \
                   vstack, ndarray, transpose, diag, kron, inf, conjugate, \
-                  int8
+                  int8, ComplexWarning
 
 import random
-from numpy.testing import *
+from numpy.testing import assert_raises, assert_equal, assert_array_equal, \
+        assert_array_almost_equal, assert_almost_equal, assert_, \
+        dec, TestCase, run_module_suite
 
 import scipy.sparse as sparse
 from scipy.sparse import csc_matrix, csr_matrix, dok_matrix, \
@@ -32,6 +34,7 @@ from scipy.sparse.linalg import splu
 
 
 warnings.simplefilter('ignore', SparseEfficiencyWarning)
+warnings.simplefilter('ignore', ComplexWarning)
 
 
 #TODO check that spmatrix( ... , copy=X ) is respected
@@ -249,7 +252,7 @@ class _TestCommon:
 
         B = A.asfptype()
         C = B.asfptype()
-        assert( B is C )
+        assert_( B is C )
 
 
     def test_mul_scalar(self):
@@ -355,7 +358,7 @@ class _TestCommon:
         """
         A = self.spmatrix([[1],[2],[3]])
 
-        assert(isspmatrix(A * array(1)))
+        assert_(isspmatrix(A * array(1)))
         assert_equal((A * array(1)).todense(), [[1],[2],[3]])
         assert_equal(A * array([1]), array([1,2,3]))
         assert_equal(A * array([[1]]), array([[1],[2],[3]]))
@@ -371,8 +374,8 @@ class _TestCommon:
         assert_equal((M * matrix([[1],[2],[3]])).shape,(4,1))
 
         #check result type
-        assert(isinstance( M * array([1,2,3]), ndarray))
-        assert(isinstance( M * matrix([1,2,3]).T, matrix))
+        assert_(isinstance( M * array([1,2,3]), ndarray))
+        assert_(isinstance( M * matrix([1,2,3]).T, matrix))
 
         #ensure exception is raised for improper dimensions
         bad_vecs = [array([1,2]), array([1,2,3,4]), array([[1],[2]]),
@@ -463,7 +466,7 @@ class _TestCommon:
 
         for b in bs:
             result = asp*b
-            assert( isinstance(result, type(b)) )
+            assert_( isinstance(result, type(b)) )
             assert_equal( result.shape, (4,2) )
             assert_equal( result, dot(a,b) )
 
@@ -555,7 +558,7 @@ class _TestCommon:
         B = A.copy()
         try:
             B[0,0] += 1
-            assert B[0,0]!=A[0,0]
+            assert_(B[0,0] != A[0,0])
         except NotImplementedError:
             # not all sparse matrices can be indexed
             pass
@@ -688,7 +691,7 @@ class _TestHorizSlicing:
             a = A[6,3:7]
         except IndexError:
             caught += 1
-        assert caught == 2
+        assert_(caught == 2)
 
 
 class _TestVertSlicing:
@@ -723,7 +726,7 @@ class _TestVertSlicing:
             a = A[6,3:7]
         except IndexError:
             caught += 1
-        assert caught == 2
+        assert_(caught == 2)
 
 
 
@@ -1295,6 +1298,34 @@ class TestDOK(_TestCommon, _TestGetSet, _TestSolve, TestCase):
         c = csr_matrix(b)
         assert_equal(A.todense(), c.todense())
 
+    def test_resize(self):
+        """A couple basic tests of the resize() method.
+        
+        resize(shape) resizes the array in-place.
+        """
+        a = dok_matrix((5,5))
+        a[:,0] = 1
+        a.resize((2,2))
+        expected1 = array([[1,0],[1,0]])
+        assert_array_equal(a.todense(), expected1)
+        a.resize((3,2))
+        expected2 = array([[1,0],[1,0],[0,0]])
+        assert_array_equal(a.todense(), expected2)
+
+
+    def test_ticket1160(self):
+        """Regression test for ticket #1160."""
+        a = dok_matrix((3,3))
+        a[0,0] = 0
+        # This assert would fail, because the above assignment would
+        # incorrectly call __set_item__ even though the value was 0.
+        assert_((0,0) not in a.keys(), "Unexpected entry (0,0) in keys")
+        
+        # Slice assignments were also affected.
+        b = dok_matrix((3,3))
+        b[:,0] = 0
+        assert_(len(b.keys())==0, "Unexpected entries in keys")
+
 
 
 class TestLIL( _TestCommon, _TestHorizSlicing, _TestVertSlicing,
