@@ -1,9 +1,12 @@
 # Author:  Travis Oliphant, 2002
 #
+# Further updates and enhancements by many SciPy developers.
+#
 
 import math
 import statlib
 import stats
+from stats import find_repeats
 import distributions
 from numpy import isscalar, r_, log, sum, around, unique, asarray
 from numpy import zeros, arange, sort, amin, amax, any, where, \
@@ -18,21 +21,14 @@ import futil
 from numpy.testing.decorators import setastest
 import warnings
 
-__all__ = ['find_repeats',
+__all__ = ['find_repeats', 'mvsdist',
            'bayes_mvs', 'kstat', 'kstatvar', 'probplot', 'ppcc_max', 'ppcc_plot',
            'boxcox_llf', 'boxcox', 'boxcox_normmax', 'boxcox_normplot',
            'shapiro', 'anderson', 'ansari', 'bartlett', 'levene', 'binom_test',
            'fligner', 'mood', 'oneway', 'wilcoxon',
-           'pdf_moments', 'pdf_fromgamma', 'pdfapprox',
-           'circmean', 'circvar', 'circstd',
+           'pdf_fromgamma', 'circmean', 'circvar', 'circstd',
           ]
 
-def find_repeats(arr):
-    """Find repeats in arr and return (repeats, repeat_count)
-    """
-    v1,v2, n = futil.dfreps(arr)
-    return v1[:n],v2[:n]
-
 
 ##########################################################
 ###  Bayesian confidence intervals for mean, variance, std
@@ -136,6 +132,40 @@ def bayes_mvs(data,alpha=0.90):
 
     return (mp,(ma,mb)),(vp,(va,vb)),(stp,(sta,stb))
 
+def mvsdist(data):
+    """Return 'frozen' distributions for mean, variance, and standard deviation of data.
+
+    Parameters
+    ----------
+    data : array-like (raveled to 1-d)
+
+    Returns
+    -------
+    mdist : "frozen" distribution object
+        Distribution object representing the mean of the data
+    vdist : "frozen" distribution object
+        Distribution object representing the variance of the data
+    sdist : "frozen" distribution object
+        Distribution object representing the standard deviation of the data
+    """
+    x = ravel(data)
+    n = len(x)
+    if (n < 2):
+        raise ValueError("Need at least 2 data-points.")
+    xbar = x.mean()
+    C = x.var()
+    if (n > 1000): # gaussian approximations for large n
+        mdist = distributions.norm(loc=xbar, scale=math.sqrt(C/n))
+        sdist = distributions.norm(loc=math.sqrt(C), scale=math.sqrt(C/(2.*n)))
+        vdist = distributions.norm(loc=C, scale=math.sqrt(2.0/n)*C)
+    else:
+        nm1 = n-1
+        fac = n*C/2.
+        val = nm1/2.
+        mdist = distributions.t(nm1,loc=xbar,scale=math.sqrt(C/nm1))
+        sdist = distributions.gengamma(val,-2,scale=math.sqrt(fac))
+        vdist = distributions.invgamma(val,scale=fac)
+    return mdist, vdist, sdist
 
 
 ################################
@@ -165,13 +195,27 @@ def bayes_mvs(data,alpha=0.90):
 ##
 
 def kstat(data,n=2):
-    """Return the nth k-statistic (1<=n<=4 so far).
+    """
+    Return the nth k-statistic (1<=n<=4 so far).
 
     The nth k-statistic is the unique symmetric unbiased estimator of the nth
-    cumulant kappa_n
+    cumulant kappa_n.
+
+    Parameters
+    ----------
+    data : array_like
+        Input array.
+    n : int, {1, 2, 3, 4}, optional
+        Default is equal to 2.
+
+    Returns
+    -------
+    kstat : float
+        The nth k-statistic.
+
     """
-    if n>4 or n<1:
-        raise ValueError, "k-statistics only supported for 1<=n<=4"
+    if n > 4 or n < 1:
+        raise ValueError("k-statistics only supported for 1<=n<=4")
     n = int(n)
     S = zeros(n+1,'d')
     data = ravel(data)
@@ -189,21 +233,35 @@ def kstat(data,n=2):
                 4*N*(N+1)*S[1]*S[3] + N*N*(N+1)*S[4]) / \
                 (N*(N-1.0)*(N-2.0)*(N-3.0))
     else:
-        raise ValueError, "Should not be here."
+        raise ValueError("Should not be here.")
 
 def kstatvar(data,n=2):
-    """Returns an unbiased estimator of the variance of the k-statistic:  n=1 or 2
+    """
+    Returns an unbiased estimator of the variance of the k-statistic.
+
+    Parameters
+    ----------
+    data : array_like
+        Input array.
+    n : int, {1, 2}, optional
+        Default is equal to 2.
+
+    Returns
+    -------
+    kstatvar : float
+        The nth k-statistic variance.
+
     """
     data = ravel(data)
     N = len(data)
-    if n==1:
+    if n == 1:
         return kstat(data,n=2)*1.0/N
-    elif n==2:
+    elif n == 2:
         k2 = kstat(data,n=2)
         k4 = kstat(data,n=4)
         return (2*k2*k2*N + (N-1)*k4)/(N*(N+1))
     else:
-        raise ValueError, "Only n=1 or n=2 supported."
+        raise ValueError("Only n=1 or n=2 supported.")
 
 
 #__all__ = ['probplot','ppcc_max','ppcc_plot','boxcox','boxcox_llf',
@@ -226,9 +284,9 @@ def probplot(x, sparams=(), dist='norm', fit=1, plot=None):
     i = arange(2,N)
     Ui[1:-1] = (i-0.3175)/(N+0.365)
     try:
-        ppf_func = eval('distributions.%s.ppf'%dist)
+        ppf_func = eval('distributions.%s.ppf' % dist)
     except AttributeError:
-        raise dist, "is not a valid distribution with a ppf."
+        raise ValueError("%s is not a valid distribution with a ppf." % dist)
     if sparams is None:
         sparams = ()
     if isscalar(sparams):
@@ -239,11 +297,11 @@ def probplot(x, sparams=(), dist='norm', fit=1, plot=None):
     res = inspect.getargspec(ppf_func)
     if not ('loc' == res[0][-2] and 'scale' == res[0][-1] and \
             0.0==res[-1][-2] and 1.0==res[-1][-1]):
-        raise ValueError, "Function has does not have default location", \
-              "and scale parameters\n  that are 0.0 and 1.0 respectively."
+        raise ValueError("Function has does not have default location "
+              "and scale parameters\n  that are 0.0 and 1.0 respectively.")
     if (len(sparams) < len(res[0])-len(res[-1])-1) or \
        (len(sparams) > len(res[0])-3):
-        raise ValueError, "Incorrect number of shape parameters."
+        raise ValueError("Incorrect number of shape parameters.")
     """
     osm = ppf_func(Ui,*sparams)
     osr = sort(x)
@@ -276,16 +334,16 @@ def ppcc_max(x, brack=(0.0,1.0), dist='tukeylambda'):
     try:
         ppf_func = eval('distributions.%s.ppf'%dist)
     except AttributeError:
-        raise dist, "is not a valid distribution with a ppf."
+        raise ValueError("%s is not a valid distribution with a ppf." % dist)
     """
     res = inspect.getargspec(ppf_func)
     if not ('loc' == res[0][-2] and 'scale' == res[0][-1] and \
             0.0==res[-1][-2] and 1.0==res[-1][-1]):
-        raise ValueError, "Function has does not have default location", \
-              "and scale parameters\n  that are 0.0 and 1.0 respectively."
+        raise ValueError("Function has does not have default location "
+              "and scale parameters\n  that are 0.0 and 1.0 respectively.")
     if (1 < len(res[0])-len(res[-1])-1) or \
        (1 > len(res[0])-3):
-        raise ValueError, "Must be a one-parameter family."
+        raise ValueError("Must be a one-parameter family.")
     """
     N = len(x)
     # compute uniform median statistics
@@ -350,18 +408,18 @@ def _boxcox_conf_interval(x, lmax, alpha):
     while (rootfunc(newlm,x,target) > 0.0) and (N < 500):
         newlm += 0.1
         N +=1
-    if (N==500):
-        raise RuntimeError, "Could not find endpoint."
+    if N == 500:
+        raise RuntimeError("Could not find endpoint.")
     lmplus = optimize.brentq(rootfunc,lmax,newlm,args=(x,target))
     newlm = lmax-0.5
     N = 0
     while (rootfunc(newlm,x,target) > 0.0) and (N < 500):
         newlm += 0.1
         N +=1
-    if (N==500):
-        raise RuntimeError, "Could not find endpoint."
-    lmminus = optimize.brentq(rootfunc,newlm,lmax,args=(x,target))
-    return lmminus,lmplus
+    if N == 500:
+        raise RuntimeError("Could not find endpoint.")
+    lmminus = optimize.brentq(rootfunc, newlm, lmax, args=(x,target))
+    return lmminus, lmplus
 
 def boxcox(x,lmbda=None,alpha=None):
     """Return a positive dataset tranformed by a Box-Cox power transformation.
@@ -375,7 +433,7 @@ def boxcox(x,lmbda=None,alpha=None):
     lambda as the third output argument.
     """
     if any(x < 0):
-        raise ValueError, "Data must be positive."
+        raise ValueError("Data must be positive.")
     if lmbda is not None:  # single transformation
         lmbda = lmbda*(x==x)
         y = where(lmbda == 0, log(x), (x**lmbda - 1)/lmbda)
@@ -470,7 +528,7 @@ def shapiro(x,a=None,reta=0):
     """
     N = len(x)
     if N < 3:
-        raise ValueError, "Data must be at least length 3."
+        raise ValueError("Data must be at least length 3.")
     if a is None:
         a = zeros(N,'f')
         init = 0
@@ -567,7 +625,8 @@ def anderson(x,dist='norm'):
 
     """
     if not dist in ['norm','expon','gumbel','extreme1','logistic']:
-        raise ValueError, "Invalid distribution."
+        raise ValueError("Invalid distribution; dist must be 'norm', "
+                            "'expon', 'gumbel', 'extreme1' or 'logistic'.")
     y = sort(x)
     xbar = np.mean(x, axis=0)
     N = len(y)
@@ -596,7 +655,7 @@ def anderson(x,dist='norm'):
         z = distributions.logistic.cdf(w)
         sig = array([25,10,5,2.5,1,0.5])
         critical = around(_Avals_logistic / (1.0+0.25/N),3)
-    elif (dist == 'gumbel') or (dist == 'extreme1'):
+    else:  # (dist == 'gumbel') or (dist == 'extreme1'):
         #the following is incorrect, see ticket:1097
 ##        def fixedsolve(th,xj,N):
 ##            val = stats.sum(xj)*1.0/N
@@ -611,43 +670,13 @@ def anderson(x,dist='norm'):
         z = distributions.gumbel_l.cdf(w)
         sig = array([25,10,5,2.5,1])
         critical = around(_Avals_gumbel / (1.0 + 0.2/sqrt(N)),3)
-    else:
-        raise ValueError("dist has to be one of 'norm','expon','logistic'",
-                         "'gumbel','extreme1'")
+
     i = arange(1,N+1)
     S = sum((2*i-1.0)/N*(log(z)+log(1-z[::-1])),axis=0)
     A2 = -N-S
     return A2, critical, sig
 
 
-def _find_repeats(arr):
-    """Find repeats in the array and return (repeats, repeat_count)
-    """
-    arr = sort(arr)
-    lastval = arr[0]
-    howmany = 0
-    ind = 1
-    N = len(arr)
-    repeat = 0
-    replist = []
-    repnum = []
-    while ind < N:
-        if arr[ind] != lastval:
-            if repeat:
-                repnum.append(howmany+1)
-                repeat = 0
-                howmany = 0
-        else:
-            howmany += 1
-            repeat = 1
-            if (howmany == 1):
-                replist.append(arr[ind])
-        lastval = arr[ind]
-        ind += 1
-    if repeat:
-        repnum.append(howmany+1)
-    return replist, repnum
-
 def ansari(x,y):
     """
     Perform the Ansari-Bradley test for equal scale parameters
@@ -686,10 +715,10 @@ def ansari(x,y):
     x,y = asarray(x),asarray(y)
     n = len(x)
     m = len(y)
-    if (m < 1):
-        raise ValueError, "Not enough other observations."
-    if (n < 1):
-        raise ValueError, "Not enough test observations."
+    if m < 1:
+        raise ValueError("Not enough other observations.")
+    if n < 1:
+        raise ValueError("Not enough test observations.")
     N = m+n
     xy = r_[x,y]  # combine
     rank = stats.rankdata(xy)
@@ -768,7 +797,7 @@ def bartlett(*args):
     """
     k = len(args)
     if k < 2:
-        raise ValueError, "Must enter at least two input sample vectors."
+        raise ValueError("Must enter at least two input sample vectors.")
     Ni = zeros(k)
     ssq = zeros(k,'d')
     for j in range(k):
@@ -799,6 +828,10 @@ def levene(*args,**kwds):
     center : {'mean', 'median', 'trimmed'}, optional
         Which function of the data to use in the test.  The default
         is 'median'.
+    proportiontocut : float, optional
+        When `center` is 'trimmed', this gives the proportion of data points
+        to cut from each end. (See `scipy.stats.trim_mean`.)
+        Default is 0.05.
 
     Returns
     -------
@@ -829,24 +862,35 @@ def levene(*args,**kwds):
               Statistical Association, 69, 364-367
 
     """
+    # Handle keyword arguments.
+    center = 'median'
+    proportiontocut = 0.05
+    for kw, value in kwds.items():
+        if kw not in ['center', 'proportiontocut']:
+            raise TypeError("levene() got an unexpected keyword argument '%s'" % kw)
+        if kw == 'center':
+            center = value
+        else:
+            proportiontocut = value
+
     k = len(args)
     if k < 2:
-        raise ValueError, "Must enter at least two input sample vectors."
+        raise ValueError("Must enter at least two input sample vectors.")
     Ni = zeros(k)
     Yci = zeros(k,'d')
-    if 'center' in kwds.keys():
-        center = kwds['center']
-    else:
-        center = 'median'
+
     if not center in ['mean','median','trimmed']:
-        raise ValueError, "Keyword argument <center> must be 'mean', 'median'"\
-              + "or 'trimmed'."
+        raise ValueError("Keyword argument <center> must be 'mean', 'median'"
+              + "or 'trimmed'.")
+
     if center == 'median':
         func = lambda x: np.median(x, axis=0)
     elif center == 'mean':
         func = lambda x: np.mean(x, axis=0)
-    else:
-        func = stats.trim_mean
+    else: # center == 'trimmed'
+        args = tuple(stats.trimboth(arg, proportiontocut) for arg in args)
+        func = lambda x: np.mean(x, axis=0)
+
     for j in range(k):
         Ni[j] = len(args[j])
         Yci[j] = func(args[j])
@@ -915,13 +959,13 @@ def binom_test(x,n=None,p=0.5):
     elif len(x) == 1:
         x = x[0]
         if n is None or n < x:
-            raise ValueError, "n must be >= x"
+            raise ValueError("n must be >= x")
         n = np.int_(n)
     else:
-        raise ValueError, "Incorrect length for x."
+        raise ValueError("Incorrect length for x.")
 
     if (p > 1.0) or (p < 0.0):
-        raise ValueError, "p must be in range [0,1]"
+        raise ValueError("p must be in range [0,1]")
 
     d = distributions.binom.pmf(x,n,p)
     rerr = 1+1e-7
@@ -963,6 +1007,10 @@ def fligner(*args,**kwds):
         keyword argument controlling which function of the data
         is used in computing the test statistic.  The default
         is 'median'.
+    proportiontocut : float, optional
+        When `center` is 'trimmed', this gives the proportion of data points
+        to cut from each end. (See `scipy.stats.trim_mean`.)
+        Default is 0.05.
 
     Returns
     -------
@@ -987,22 +1035,32 @@ def fligner(*args,**kwds):
            71(353), 210-213.
 
     """
+    # Handle keyword arguments.
+    center = 'median'
+    proportiontocut = 0.05
+    for kw, value in kwds.items():
+        if kw not in ['center', 'proportiontocut']:
+            raise TypeError("fligner() got an unexpected keyword argument '%s'" % kw)
+        if kw == 'center':
+            center = value
+        else:
+            proportiontocut = value
+
     k = len(args)
     if k < 2:
-        raise ValueError, "Must enter at least two input sample vectors."
-    if 'center' in kwds.keys():
-        center = kwds['center']
-    else:
-        center = 'median'
+        raise ValueError("Must enter at least two input sample vectors.")
+
     if not center in ['mean','median','trimmed']:
-        raise ValueError, "Keyword argument <center> must be 'mean', 'median'"\
-              + "or 'trimmed'."
+        raise ValueError("Keyword argument <center> must be 'mean', 'median'"
+              + "or 'trimmed'.")
+
     if center == 'median':
         func = lambda x: np.median(x, axis=0)
     elif center == 'mean':
         func = lambda x: np.mean(x, axis=0)
-    else:
-        func = stats.trim_mean
+    else: # center == 'trimmed'
+        args = tuple(stats.trimboth(arg, proportiontocut) for arg in args)
+        func = lambda x: np.mean(x, axis=0)
 
     Ni = asarray([len(args[j]) for j in range(k)])
     Yci = asarray([func(args[j]) for j in range(k)])
@@ -1015,15 +1073,14 @@ def fligner(*args,**kwds):
         allZij.extend(list(Zij[i]))
         g.append(len(allZij))
 
-    a = distributions.norm.ppf(stats.rankdata(allZij)/(2*(Ntot+1.0)) + 0.5)
+    ranks = stats.rankdata(allZij)
+    a = distributions.norm.ppf(ranks/(2*(Ntot+1.0)) + 0.5)
 
     # compute Aibar
     Aibar = _apply_func(a,g,sum) / Ni
     anbar = np.mean(a, axis=0)
     varsq = np.var(a,axis=0, ddof=1)
-
     Xsq = sum(Ni*(asarray(Aibar)-anbar)**2.0,axis=0)/varsq
-
     pval = distributions.chi2.sf(Xsq,k-1) # 1 - cdf
     return Xsq, pval
 
@@ -1064,8 +1121,8 @@ def mood(x,y):
     m = len(y)
     xy = r_[x,y]
     N = m+n
-    if (N < 3):
-        raise ValueError, "Not enough observations."
+    if N < 3:
+        raise ValueError("Not enough observations.")
     ranks = stats.rankdata(xy)
     Ri = ranks[:n]
     M = sum((Ri - (N+1.0)/2)**2,axis=0)
@@ -1073,13 +1130,13 @@ def mood(x,y):
     mnM = n*(N*N-1.0)/12
     varM = m*n*(N+1.0)*(N+2)*(N-2)/180
     z = (M-mnM)/sqrt(varM)
-    
+
     # Numerically better than p = norm.cdf(x); p = min(p, 1 - p)
     if z > 0:
         pval = distributions.norm.sf(z)
     else:
         pval = distributions.norm.cdf(z)
-    
+
     # Account for two-sidedness
     pval *= 2.
     return z, pval
@@ -1098,7 +1155,7 @@ def oneway(*args,**kwds):
     """
     k = len(args)
     if k < 2:
-        raise ValueError, "Must enter at least two input sample vectors."
+        raise ValueError("Must enter at least two input sample vectors.")
     if 'equal_var' in kwds.keys():
         if kwds['equal_var']: evar = 1
         else: evar = 0
@@ -1124,7 +1181,6 @@ def oneway(*args,**kwds):
     return F, pval
 
 
-
 def wilcoxon(x,y=None):
     """
     Calculate the Wilcoxon signed-rank test
@@ -1166,7 +1222,7 @@ def wilcoxon(x,y=None):
     else:
         x, y = map(asarray, (x, y))
         if len(x) <> len(y):
-            raise ValueError, 'Unequal N in wilcoxon.  Aborting.'
+            raise ValueError('Unequal N in wilcoxon.  Aborting.')
         d = x-y
     d = compress(not_equal(d,0),d,axis=-1) # Keep all non-zero differences
     count = len(d)
@@ -1202,42 +1258,6 @@ def _hermnorm(N):
         plist[n] = plist[n-1].deriv() - poly1d([1,0])*plist[n-1]
     return plist
 
-@np.lib.deprecate(message="""
-scipy.stats.pdf_moments is broken. It will be removed from scipy in 0.9
-unless it is fixed.
-""")
-def pdf_moments(cnt):
-    """Return the Gaussian expanded pdf function given the list of central
-    moments (first one is mean).
-    """
-    N = len(cnt)
-    if N < 2:
-        raise ValueError, "At least two moments must be given to" + \
-              "approximate the pdf."
-    totp = poly1d(1)
-    sig = sqrt(cnt[1])
-    mu = cnt[0]
-    if N > 2:
-        Dvals = _hermnorm(N+1)
-    for k in range(3,N+1):
-        # Find Ck
-        Ck = 0.0
-        for n in range((k-3)/2):
-            m = k-2*n
-            if m % 2: # m is odd
-                momdiff = cnt[m-1]
-            else:
-                momdiff = cnt[m-1] - sig*sig*scipy.factorial2(m-1)
-            Ck += Dvals[k][m] / sig**m * momdiff
-        # Add to totp
-        totp = totp +  Ck*Dvals[k]
-
-    def thisfunc(x):
-        xn = (x-mu)/sig
-        return totp(xn)*exp(-xn*xn/2.0)/sqrt(2*pi)/sig
-    return thisfunc
-
-
 def pdf_fromgamma(g1,g2,g3=0.0,g4=None):
     if g4 is None:
         g4 = 3*g2*g2
@@ -1261,32 +1281,25 @@ def pdf_fromgamma(g1,g2,g3=0.0,g4=None):
         return totp(xn)*exp(-xn*xn/2.0)
     return thefunc
 
-@np.lib.deprecate(message="""
-scipy.stats.pdfapprox is broken. It will be removed from scipy in 0.9
-unless it is fixed.
-""")
-def pdfapprox(samples):
-    """Return a function that approximates the pdf of a set of samples
-    using a Gaussian expansion computed from the mean, variance, skewness
-    and Fisher's kurtosis.
-    """
-    # Estimate mean, variance, skewness and kurtosis
-    mu,sig,sk,kur = stats.describe(samples)[2:]
-    # Get central moments
-    cnt = [None]*4
-    cnt[0] = mu
-    cnt[1] = sig*sig
-    cnt[2] = sk * sig**1.5
-    cnt[3] = (kur+3.0) * sig**2.0
-    return pdf_moments(cnt)
-    #g2 = (1.0/sig)**2.0
-    #g1 = mu / sig**3.0
-    #g3 = sk / sig**3.5
-    #g4 = (kur+3.0) / sig**4.0
-    #return pdf_fromgamma(g1, g2, g3, g4)
-
 def circmean(samples, high=2*pi, low=0):
-    """Compute the circular mean for samples assumed to be in the range [low to high]
+    """
+    Compute the circular mean for samples assumed to be in the range
+    [low to high].
+
+    Parameters
+    ----------
+    samples : array_like
+        Input array.
+    low : float or int, optional
+        Low boundary for circular mean range.  Default is 0.
+    high : float or int, optional
+        High boundary for circular mean range.  Default is 2*pi.
+
+    Returns
+    -------
+    circmean : float
+        Circular mean.
+
     """
     ang = (samples - low)*2*pi / (high-low)
     res = angle(np.mean(exp(1j*ang), axis=0))
@@ -1295,7 +1308,24 @@ def circmean(samples, high=2*pi, low=0):
     return res*(high-low)/2.0/pi + low
 
 def circvar(samples, high=2*pi, low=0):
-    """Compute the circular variance for samples assumed to be in the range [low to high]
+    """
+    Compute the circular variance for samples assumed to be in the range
+    [low to high].
+
+    Parameters
+    ----------
+    samples : array_like
+        Input array.
+    low : float or int, optional
+        Low boundary for circular variance range.  Default is 0.
+    high : float or int, optional
+        High boundary for circular variance range.  Default is 2*pi.
+
+    Returns
+    -------
+    circvar : float
+        Circular variance.
+
     """
     ang = (samples - low)*2*pi / (high-low)
     res = np.mean(exp(1j*ang), axis=0)
@@ -1303,7 +1333,24 @@ def circvar(samples, high=2*pi, low=0):
     return ((high-low)/2.0/pi)**2 * V
 
 def circstd(samples, high=2*pi, low=0):
-    """Compute the circular standard deviation for samples assumed to be in the range [low to high]
+    """
+    Compute the circular standard deviation for samples assumed to be in the
+    range [low to high].
+
+    Parameters
+    ----------
+    samples : array_like
+        Input array.
+    low : float or int, optional
+        Low boundary for circular standard deviation range.  Default is 0.
+    high : float or int, optional
+        High boundary for circular standard deviation range.  Default is 2*pi.
+
+    Returns
+    -------
+    circstd : float
+        Circular standard deviation.
+
     """
     ang = (samples - low)*2*pi / (high-low)
     res = np.mean(exp(1j*ang), axis=0)
